//
// Title:   srf2sfl.cpp
// Created: Mon Feb 24 19:00:00 2003
// Authors: Tamy Boubekeur
//
//
// This file is a tool useful for Cotex.
// See http://www.cotex.fr.st for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact tamyboubekeur@libertysurf.fr if any conditions of this
// licensing are not clear to you.
//

#include "pointshop_src/include/sfl.h"

#include <iostream.h>
#include <fstream.h> 
#include <stdlib.h>

/**
 * srf2sf is a small filter, which converts files from .srf to .sfl 
 * (PointShop format - http://graphics.ethz.ch/pointshop3d/). It's 
 * part of the CoTeX project (http://www.cotex.fr.st).
 *
 * \todo the same with 1 ifstream.
 */
static const unsigned int MAX_COMMENT = 256;
static const double DEFAULT_FACTOR = 1.0;
static const double DEFAULT_RADIUS = 1.0;

char dummy[MAX_COMMENT];

using namespace std;

/// The Right Command Line

void usage (char *com){
  cerr << "Usage : " << com << " <filein.srf> <fileout.sfl> [factor]" << endl;
  exit (EXIT_FAILURE);
}

/// If there's no radius, return the DEFAULT_RADIUS
float readHeader (ifstream& input) {
  char c;
  input.get (c);
  input.putback (c); // back to the stream
  if (c != '#') // no comment
    return DEFAULT_RADIUS; 
  
  input.get (c);
  float res; 
  input >> res;
  input.getline (dummy, MAX_COMMENT);
  
  // read the rest of the com.
  input.get (c);
  input.putback (c); // back to the stream
  while (c == '#') {
    input.getline (dummy, MAX_COMMENT);
    input.get (c);
    input.putback (c); // back to the stream
  }
  return res;
}

int  main(int argc, char* argv[]) {    
  if (argc < 3 || argc > 4)
    usage (argv[0]);

  float factor = DEFAULT_FACTOR;

  if (argc == 4) { // read factor
    char * endptr = new char[1];
    factor = strtod (argv[3], &endptr);
    if (*endptr != '\0') { 
      cerr << "Bad command : factor is not a real" << endl; 
      usage (argv[0]);
      exit (EXIT_FAILURE); 
    }
    delete [] endptr;
  }
      
      
  ifstream input0(argv[1], ios::in);
  sfl::OutStream  *out;   
  int cpt = 0;
  double tmp;
  
  if (!input0) { 
    cerr << " Error : Opening file" << argv[1] << endl; 
    exit (EXIT_FAILURE); 
  } 
  
  // count the number of surfels in the file (neede by sfl)
  cout << "Convertion, please wait... " << flush;
  
  readHeader (input0); // actually, the radius
  while (!input0.eof()) {
    input0 >> tmp;
    cpt ++;
  }
  input0.close ();
  /*
  if (cpt % 9 != 0) { 
    cerr << " Error : Bad file " << argv[1] << endl; 
    exit (EXIT_FAILURE); 
  } 
  */

  uint32 nPoints = cpt / 9; // 9 integer for each surfel in .srf
  
  ifstream input(argv[1], ios::in);
  if (!input) { 
    cout << "error!" << endl;
    cerr << "  Error : Opening file" << endl; 
    exit (EXIT_FAILURE); 
  } 
    
  if ((out = sfl::OutStream::open(argv[2])) != NULL) {    
    
    // defined which property will be saved
    uint32  sflProps = (SFLPROP_POSITION |
			SFLPROP_NORMAL |
			SFLPROP_RADIUS | // not necessary (don't appear in srf)
                        SFLPROP_DIFFUSE_COLOR | 
                        SFLPROP_COLOR_MODEL_RGB);
    
    // Header
    out->beginHeader();
    out->setTitle(argv[1]);
    out->endHeader();
    
    // starting with data
    out->beginSurfelSetList(1);
    out->beginSurfelSet(sflProps);
    out->beginResolution(nPoints);

    float x, y, z, nx, ny, nz, r, g, b, radius;
    radius = readHeader (input);
    cout << endl << "Radius readed : " << radius << endl;
    
    for (int i = 0; i < nPoints; i++) {
      // read and write a surfel
      input >> x >> y >> z >> nx >> ny >> nz >> r >> g >> b;
      out->beginSurfel();
      out->dumpSurfelPosition3 (factor * x, factor * y, factor * z);
      out->dumpSurfelNormal3 (nx, ny, nz);
      out->dumpSurfelRadius (radius); 
      out->dumpSurfelColorRGBf (sfl::DIFFUSE, r, g, b);
      out->endSurfel();     
    } 
    
    // end of surfels's data
    out->endResolution();
    out->endSurfelSet(); 
    out->endSurfelSetList();
    
    // close streams
    input.close ();
    sfl::OutStream::close (out);
    cout << "finished!" << endl;
  }  
  return EXIT_SUCCESS;
}


